# Partial Reference https://github.com/huggingface/transformers/utils/hub

import requests
import os
from pathlib import Path
from typing import Dict, List, Optional, Union
from urllib.parse import urlparse

ENV_VARS_TRUE_VALUES = {"1", "ON", "YES", "TRUE"}
_is_offline_mode = (
    True
    if os.environ.get("TRANSFORMERS_OFFLINE", "0").upper() in ENV_VARS_TRUE_VALUES
    else False
)
_CACHED_NO_EXIST = object()

cache_root = os.path.join(os.path.expanduser("~"), ".cache/googledriver")
if not os.path.exists(cache_root):
    try:
        os.makedirs(cache_root)
    except OSError:
        pass

DEFAULT_CACHE_FOLDER = cache_root


def is_offline_mode():
    return _is_offline_mode


def download(URL: str, save_path: str, cached_filename) -> str:
    """Just put the full file path in the local area and the Google Drive file path accessible to everyone, and you can download it.

    :param URL: Google Drive file path accessible to everyone
    :type URL: str
    :param save_path: Full file name to save to local storage
    :type save_path: str
    :param cached_filename: _description_, defaults to None
    :type cached_filename: File save name if you want to use it as a cache, optional
    :return: saved path
    :rtype: str
    """

    session = requests.Session()
    response = session.get(URL, stream=True)
    token = get_token(response)
    if token:
        response = session.get(URL, stream=True)

    if cached_filename is not None:
        cached = try_to_load_from_cache(cached_filename, None)
        if cached == _CACHED_NO_EXIST:
            cached_path = os.path.join(DEFAULT_CACHE_FOLDER, cached_filename)
            save_file(response, cached_path)
            saved_path = cached_path
    else:
        save_file(response, save_path)
        saved_path = save_path

    return save_path


def get_token(response: str) -> str:
    """The response to the Google Drive request is stored in the token.

    :param response: Responding to Google Drive requests
    :type response: str
    :return: Returns if a warning occurs
    :rtype: str
    """
    for k, v in response.cookies.items():
        if k.startswith("download_warning"):
            return v


def save_file(response: str, save_path: str) -> None:
    """Save the file to local storage in response to the request.

    :param response: Responding to Google Drive requests
    :type response: str
    :param save_path: Full file name to save to local storage
    :type save_path: str
    """
    CHUNK_SIZE = 99999999
    with open(save_path, "wb") as f:
        for chunk in response.iter_content(CHUNK_SIZE):
            if chunk:
                f.write(chunk)

    return None


def try_to_load_from_cache(
    cached_filename: str, cache_dir: Union[str, Path, None] = None,
) -> Optional[str]:

    if cache_dir is None:
        cache_dir = DEFAULT_CACHE_FOLDER

    cached_file = os.path.join(cache_dir, cached_filename)
    if not os.path.isfile(cached_file):
        return _CACHED_NO_EXIST

    return cached_file if os.path.isfile(cached_file) else None


def cached_file(
    cached_filename: str,
    cache_dir: Optional[Union[str, os.PathLike]] = None,
    local_files_only: bool = False,
):

    if is_offline_mode() and not local_files_only:
        print("OFF-LINE MODE: forcing local_files_only")
        local_files_only = True

    if cache_dir is None:
        cache_dir = DEFAULT_CACHE_FOLDER

    if isinstance(cache_dir, Path):
        cache_dir = str(cache_dir)

    try:
        # Load from URL or cache if already cached
        resolved_file = os.path.join(DEFAULT_CACHE_FOLDER, cached_filename)
    except Exception as e:
        print(e)

    return resolved_file


def get_cachefile_from_driver(
    URL: Union[str, os.PathLike],
    filename: str,
    cache_dir: Optional[Union[str, os.PathLike]] = None,
):
    download(URL, "", filename)

    return cached_file(
        cached_filename=filename,
        cache_dir=cache_dir,
        _raise_exceptions_for_missing_entries=False,
        _raise_exceptions_for_connection_errors=False,
    )



from __future__ import print_function

import argparse
import os.path
import re
import sys
import warnings

import six

from . import __version__
from .downloader import download
from .folder_downloader import download_folder
from .folder_downloader import MAX_DOWNLOAD_LIMIT


class _ShowVersionAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        print(
            "google0driver {ver} at {pos}".format(
                ver=__version__, pos=os.path.dirname(os.path.dirname(__file__))
            )
        )
        parser.exit()


def file_size(argv):
    if argv is not None:
        m = re.match(r"([0-9]+)(GB|MB|KB|B)", argv)
        if not m:
            raise TypeError
        size, unit = m.groups()
        size = float(size)
        if unit == "KB":
            size *= 1024
        elif unit == "MB":
            size *= 1024 ** 2
        elif unit == "GB":
            size *= 1024 ** 3
        elif unit == "B":
            pass
        return size


def main():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument(
        "-V",
        "--version",
        action=_ShowVersionAction,
        help="display version of google-driver package.",
        nargs=0,
    )
    parser.add_argument(
        "--url", help="url(permission must be checked) to download from"
    )

    parser.add_argument("-O", "--output", help="output path")

    parser.add_argument(
        "--folder",
        action="store_true",
        help="download entire folder instead of a single file "
        "(max {max} files per folder)".format(max=MAX_DOWNLOAD_LIMIT),
    )

    parser.add_argument(
        "--save_path", "-O", "--output", help="full file or folder path",
    )

    args = parser.parse_args()
    url = None

    if args.output == "-":
        if six.PY3:
            args.output = sys.stdout.buffer
        else:
            args.output = sys.stdout

    if args.folder:
        filenames = download_folder(URL=url, save_path=None, cached_filename=None)
        success = filenames is not None
    else:
        filename = download(URL=url, save_path=None, cached=None)
        success = filename is not None

    if not success:
        sys.exit(1)


if __name__ == "__main__":
    main()



import re

from setuptools import find_packages
from setuptools import setup


def get_version():
    filename = "googledriver/__init__.py"
    with open(filename) as f:
        match = re.search(r"""^__version__ = ['"]([^'"]*)['"]""", f.read(), re.M)
    if not match:
        raise RuntimeError("{} doesn't contain __version__".format(filename))
    version = match.groups()[0]
    return version


def get_long_description():
    with open("README.md") as f:
        long_description = f.read()

    try:
        import github2pypi  # NOQA

        return github2pypi.replace_url(slug="wkentaro/gdown", content=long_description)
    except Exception:
        return long_description


version = get_version()


setup(
    name="googledriver",
    version="0.1.7",
    author="parkminwoo",
    author_email="parkminwoo1991@gmail.com",
    description="The Python package google drive facilitates access to files uploaded to Google Drive.",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/DSDanielPark/google-driver",
    packages=find_packages(exclude=["github2pypi"]),
    python_requires=">=3.6",
    install_requires=["filelock", "requests[socks]", "six", "tqdm", "beautifulsoup4",],
    keywords="Download, Googledrive",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    entry_points={"console_scripts": ["googldriver=googldriver.cli:main"]},
)



# utf-8
# flake8: noqa

from googledriver.downloader import (
    download,
    get_token,
    save_file,
    is_offline_mode,
    try_to_load_from_cache,
    cached_file,
    get_cachefile_from_driver,
)
from googledriver.folder_downloader import download_folder

__all__ = [
    download,
    get_token,
    save_file,
    is_offline_mode,
    try_to_load_from_cache,
    cached_file,
    get_cachefile_from_driver,
    download_folder,
]

__version__ = "0.1.7"
__author__ = "MinWoo Park <parkminwoo1991@gmail.com>"



# -*- encoding: utf-8 -*-
# Partial Reference https://github.com/wkentaro/gdown


from __future__ import print_function
from googledriver import download
import itertools
import json
import os
import os.path as ospath
import re
import sys
import textwrap
import warnings
import bs4
import requests


MAX_DOWNLOAD_LIMIT = 50


class _GoogleDriveObj(object):
    FOLDER = "application/vnd.google-apps.folder"

    def __init__(self, id, name, type, children=None):
        self.id = id
        self.name = name
        self.type = type
        self.children = children if children is not None else []

    def is_folder(self):
        return self.type == self.FOLDER


def _parse_google_drive_obj(URL, content):

    folder_soup = bs4.BeautifulSoup(content, features="html.parser")

    encoded_data = None
    for script in folder_soup.select("script"):
        inner_html = script.decode_contents()

        if "_DRIVE_ivd" in inner_html:
            regex_iter = re.compile(r"'((?:[^'\\]|\\.)*)'").finditer(inner_html)
            try:
                encoded_data = next(itertools.islice(regex_iter, 1, None)).group(1)
            except StopIteration:
                raise RuntimeError("Couldn't find the folder encoded JS string")
            break

    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", category=DeprecationWarning)
        decoded = encoded_data.encode("utf-8").decode("unicode_escape")
    folder_arr = json.loads(decoded)

    folder_contents = [] if folder_arr[0] is None else folder_arr[0]

    sep = " - "
    splitted = folder_soup.title.contents[0].split(sep)
    if len(splitted) >= 2:
        name = sep.join(splitted[:-1])
    else:
        raise RuntimeError(
            "file/folder name cannot be extracted: {}".format(
                folder_soup.title.contents[0]
            )
        )

    google_drive_file = _GoogleDriveObj(
        id=URL.split("/")[-1], name=name, type=_GoogleDriveObj.FOLDER,
    )

    id_name_type_iter = [
        (e[0], e[2].encode("raw_unicode_escape").decode("utf-8"), e[3])
        for e in folder_contents
    ]

    return google_drive_file, id_name_type_iter


def _get_google_drive_objs(URL,):

    status_code = True

    if "?" in URL:
        URL += "&hl=en"
    else:
        URL += "?hl=en"

    session = requests.Session()

    try:
        res = session.get(URL)
    except requests.exceptions.ProxyError as e:
        print("Proxy error:", session.proxies, file=sys.stderr)
        print(e, file=sys.stderr)
        return False, None

    if res.status_code != 200:
        return False, None

    google_drive_file, id_name_type_iter = _parse_google_drive_obj(
        URL=URL, content=res.text,
    )

    for child_id, child_name, child_type in id_name_type_iter:
        if child_type != _GoogleDriveObj.FOLDER:
            google_drive_file.children.append(
                _GoogleDriveObj(id=child_id, name=child_name, type=child_type,)
            )
            if not status_code:
                return status_code, None
            continue

        status_code, child = _get_google_drive_objs(
            URL="https://drive.google.com/drive/folders/" + child_id,
        )
        if not status_code:
            return status_code, None
        google_drive_file.children.append(child)

    return status_code, google_drive_file


def _get_directory_structure(google_drive_file, previous_path):

    directory_structure = []
    for file in google_drive_file.children:
        file.name = file.name.replace(ospath.sep, "_")
        if file.is_folder():
            directory_structure.append((None, ospath.join(previous_path, file.name)))
            for i in _get_directory_structure(
                file, ospath.join(previous_path, file.name)
            ):
                directory_structure.append(i)
        elif not file.children:
            directory_structure.append((file.id, ospath.join(previous_path, file.name)))
    return directory_structure


cache_root = os.path.join(os.path.expanduser("~"), ".cache\\googledriver")
if not os.path.exists(cache_root):
    try:
        os.makedirs(cache_root)
    except OSError:
        pass


def download_folder(URL=None, save_path=None, cached=None):

    session = requests.Session()

    try:
        status_code, google_drive_file = _get_google_drive_objs(URL,)
    except RuntimeError as e:
        print("Failed to download:", file=sys.stderr)
        error = "\n".join(textwrap.wrap(str(e)))
        print("\n", error, "\n", file=sys.stderr)
        return

    if not status_code:
        return status_code
    if save_path is None:
        save_path = os.getcwd() + ospath.sep
    if save_path.endswith(ospath.sep):
        root_folder = ospath.join(save_path, google_drive_file.name)
    else:
        root_folder = save_path

    if cached == True:
        root_folder = cache_root

    directory_structure = _get_directory_structure(google_drive_file, root_folder)
    if not ospath.exists(root_folder):
        os.makedirs(root_folder)

    filenames = []
    for file_id, file_path in directory_structure:
        if file_id is None:
            if not ospath.exists(file_path):
                os.makedirs(file_path, exist_ok=True)
            continue

        filename = download(
            URL="https://drive.google.com/uc?id=" + file_id,
            save_path=file_path,
            cached_filename=None,
        )

        if filename is None:
            print("Failed to download", file=sys.stderr)

        filenames.append(filename)

    if cached == True:
        filenames = cache_root

    return filenames
